@@ -1,9 +1,10 @@
 from pathlib import Path
 from .tools import _search_path
 
-VERSIONS = [7.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0]
+VERSIONS = [7.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0, 14.0, 15.0, 16.0, 17.0, 18.0]
 YAML_VERSION = '3.7'
 DEFAULT_IMAGES_REPO = "https://github.com/marcwimmer/wodoo-images"
+IMAGES_REPO_BRANCH = "2024-09"
 
 default_dirs = {
     'admin': 'admin',
@@ -22,11 +23,12 @@ default_dirs = {
     'odoo_tools': '$odoo_home',
     'odoo_data_dir': "~/.odoo/files",
     'user_conf_dir': "~/.odoo",
-    'cicd_delegator': '~/.odoo/cicd_delegator',
+    'delegator': '~/.odoo/delegator',
     'images': '~/.odoo/images',
 }
 
 default_files = {
+    'queuejob_channels_file': '${run}/queue-job-channels.txt',
     'odoo_docker_file': '${run}/Dockerfile.odoo',
     'after_reload_script': "/usr/local/bin/after-odoo-reload.sh",
     'after_up_script': "/usr/local/bin/after-odoo-up.sh",
@@ -36,6 +38,7 @@ default_files = {
     'project_docker_compose.home': "~/.odoo/docker-compose.yml",
     'project_docker_compose.home.project': "~/.odoo/docker-compose.${project_name}.yml",
     'project_docker_compose.local': "${working_dir}/.odoo/docker-compose.${project_name}.yml",
+    'docker_bin': _search_path('docker'),
     'docker_compose': '${run}/docker-compose.yml',
     'docker_compose_bin': _search_path('docker-compose'),
     'debugging_template_withports': 'config/template_withports.yml',
@@ -53,12 +56,13 @@ default_files = {
     'native_bin_restore_dump': "${run_native_bin_dir}/restore-db",
     'native_collected_requirements_from_modules': "${run_native_bin_dir}/customs-requirements.txt",
     'start-dev': '~/.odoo/start-dev',
-    'cicd_delegator_registry': '${cicd_delegator}/registry.json',
+    'delegator_registry': '${delegator}/registry.json',
     'pgcli_history': '${run}/pgcli_history',
 }
 
 default_commands = {
     'dc': ['${docker_compose_bin}', "-p", "${project_name}", "-f",  "${docker_compose}"],
+    'dc2': ['${docker_bin}', 'compose', "-p", "${project_name}", "-f",  "${docker_compose}"],
 }
 
 FILE_DIRHASHES = '.dirhashes'
\ No newline at end of file
