@@ -1,5 +1,6 @@
 import traceback
 from .tools import bashfind
+import json
 import arrow
 import threading
 from tabulate import tabulate
@@ -24,6 +25,7 @@ import copy
 import click
 from . import module_tools
 from . import tools
+from .tools import update_setting
 from .tools import __replace_all_envs_in_str
 from .tools import __running_as_root_or_sudo
 from .tools import _makedirs
@@ -32,16 +34,23 @@ from .tools import whoami
 from .tools import abort
 from .tools import _get_version
 from .tools import rsync
+from .tools import __read_file
+from .tools import get_docker_version
+from .tools import _parse_yaml
 from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
 from .odoo_config import MANIFEST
 from .tools import execute_script
 from .tools import ensure_project_name
+from .tools import update_docker_service
 
 import inspect
 import os
 from pathlib import Path
-current_dir = Path(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))))
+
+current_dir = Path(
+    os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
+)
 
 
 @cli.group(cls=AliasedGroup)
@@ -115,6 +124,7 @@ def _get_arch():
 @click.option(
     "--docker-compose", help="additional docker-compose files, separated by colon :"
 )
+@click.option("-s", "--include-src", help="include source")
 @pass_config
 @click.pass_context
 def do_reload(
@@ -132,6 +142,7 @@ def do_reload(
     no_update_images,
     docker_compose,
     no_gimera_apply,
+    include_src,
 ):
     from .myconfigparser import MyConfigParser
     from .module_tools import NotInAddonsPath
@@ -149,6 +160,7 @@ def do_reload(
     if SETTINGS_FILE and SETTINGS_FILE.exists():
         SETTINGS_FILE.unlink()
 
+
     additional_config_file = None
     try:
         if additional_config:
@@ -194,6 +206,7 @@ def do_reload(
                 additional_config,
                 additional_docker_configuration_files=docker_compose,
                 no_gimera_apply=no_gimera_apply,
+                include_src=include_src,
             )
         except NotInAddonsPath as ex:
             abort(str(ex))
@@ -220,6 +233,7 @@ def internal_reload(
     additional_config=None,
     additional_docker_configuration_files=None,
     no_gimera_apply=False,
+    include_src=False,
 ):
     ensure_project_name(config)
     additional_docker_configuration_files = additional_docker_configuration_files or []
@@ -229,7 +243,7 @@ def internal_reload(
         "demo": demo,
         "LOCAL_SETTINGS": "1" if local else "0",
         "CUSTOMS_DIR": config.WORKING_DIR,
-        "WODOO_VERSION": _get_version(),
+        "QUEUEJOB_CHANNELS_FILE": config.files["queuejob_channels_file"],
     }
     if devmode:
         defaults["DEVMODE"] = 1
@@ -261,11 +275,14 @@ def internal_reload(
     os.environ[KEY_ODOO_VERSION] = ODOO_VERSION
     defaults[KEY_ODOO_VERSION] = ODOO_VERSION
 
+    before_reload(config)
+
     # assuming we are in the odoo directory
     _do_compose(
         **defaults,
         additional_docker_configuration_files=additional_docker_configuration_files,
         no_gimera_apply=no_gimera_apply,
+        include_src=include_src,
     )
 
     _execute_after_reload(config)
@@ -283,7 +300,7 @@ def _set_defaults(config, defaults):
     defaults["HOST_RUN_DIR"] = config.HOST_RUN_DIR
     defaults["NETWORK_NAME"] = config.project_name
     defaults["project_name"] = config.project_name
-
+    defaults["WODOO_VERSION"] = _get_version()
 
 def _do_compose(
     config,
@@ -291,6 +308,7 @@ def _do_compose(
     demo=False,
     additional_docker_configuration_files=None,
     no_gimera_apply=False,
+    include_src=False,
     **forced_values,
 ):
     """
@@ -323,7 +341,7 @@ def _do_compose(
     _prepare_yml_files_from_template_files(
         config, additional_docker_configuration_files
     )
-    _merge_odoo_dockerfile(config)
+    _merge_odoo_dockerfile(config, include_src=include_src)
 
     click.echo("Built the docker-compose file.")
 
@@ -340,30 +358,26 @@ def _download_images(config, images_url):
                 config.dirs["images"],
             ]
         )
-    # subprocess.check_call(
-    #     [
-    #         "git",
-    #         "config",
-    #         "--global",
-    #         "--add",
-    #         "safe.directory",
-    #         str(config.dirs["images"]),
-    #     ],
-    #     cwd=config.dirs["images"],
-    # )
     current_branch = subprocess.check_output(
         ["git", "rev-parse", "--abbrev-ref", "HEAD"],
         encoding="utf8",
         cwd=config.dirs["images"],
     ).strip()
-    if config.ODOO_IMAGES_BRANCH and config.ODOO_IMAGES_BRANCH != current_branch:
-        subprocess.check_call(["git", "checkout", config.ODOO_IMAGES_BRANCH])
+    effective_branch = config.ODOO_IMAGES_BRANCH or consts.IMAGES_REPO_BRANCH
+
+    if effective_branch and effective_branch != current_branch:
+        subprocess.check_call(
+            ["git", "fetch"], cwd=config.dirs["images"]
+        )
+        subprocess.check_call(
+            ["git", "checkout", effective_branch], cwd=config.dirs["images"]
+        )
 
     if subprocess.check_output(
         ["git", "remote"], encoding="utf8", cwd=config.dirs["images"]
     ).strip():
         trycount = 0
-        for i in range(10):
+        for _ in range(10):
             trycount += 1
             try:
                 subprocess.check_call(["git", "pull"], cwd=config.dirs["images"])
@@ -505,7 +519,7 @@ def _execute_after_settings(config):
         )
         module = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(module)
-        module.after_settings(settings)
+        module.after_settings(settings, config=config)
         settings.write()
 
 
@@ -641,7 +655,7 @@ def post_process_complete_yaml_config(config, yml):
     complete configuration.
     """
 
-    yml["version"] = config.YAML_VERSION
+    _set_yaml_version(yml, config)
 
     # remove restart policies, if not restart allowed:
     if not config.restart_containers:
@@ -869,6 +883,13 @@ def _explode_referenced_machines(contents):
                     content["services"][to_explode] = src
 
 
+def _set_yaml_version(content, config):
+    if get_docker_version()[0] < 26:
+        content["version"] = config.YAML_VERSION
+    else:
+        content.pop("version", None)
+
+
 def _apply_variables(config, contents, env):
     import yaml
 
@@ -887,7 +908,7 @@ def _apply_variables(config, contents, env):
         for networkname, network in content.get("networks", {}).items():
             default_network["networks"][networkname] = network
 
-        content["version"] = config.YAML_VERSION
+        _set_yaml_version(content, config)
 
         # set settings environment and the override settings after that
         __set_environment_in_services(content)
@@ -959,7 +980,9 @@ def _use_file(config, path):
                 ".run_" in x for x in path.parts
             ):
                 return False
-            if not any(".run_" in x for x in path.parts):
+            if not any(".run_" in x for x in path.parts) and not any(
+                ".!run_" in x for x in path.parts
+            ):
                 # allower postgres/docker-compose.yml
                 return True
 
@@ -980,7 +1003,11 @@ def _use_file(config, path):
                 return True
 
         # requires general run:
-        if getattr(config, f"run_{path.parent.name}") or "run_" in path.name:
+        if (
+            getattr(config, f"run_{path.parent.name}")
+            or "run_" in path.name
+            or "!run_" in path.name
+        ):
             run = list(
                 filter(
                     lambda x: x.startswith("run_"),
@@ -999,9 +1026,9 @@ def _use_file(config, path):
             )
             for run in run:
                 if not getattr(config, run):
-                    return True
+                    return False
                 if getattr(config, run.lower().replace("run_", "")):
-                    return True
+                    return False
             return False
 
         if path.absolute() == config.files["docker_compose"].absolute():
@@ -1020,7 +1047,7 @@ def _use_file(config, path):
     return res
 
 
-def _merge_odoo_dockerfile(config):
+def _merge_odoo_dockerfile(config, include_src):
     """
     If customs contains dockerfile, then append their execution
     in the main dockerfile
@@ -1057,10 +1084,16 @@ def _merge_odoo_dockerfile(config):
             content = content.replace("${PROJECT_NAME}", config.project_name)
             file.write_text(content)
 
+        # append common docker config
+        odoo_docker_file = config.files["odoo_docker_file"]
+        common = config.dirs["images"] / "odoo" / "config" / "common.docker"
+        odoo_docker_file.write_text(
+            odoo_docker_file.read_text() + "\n" + common.read_text()
+        )
+
         # include source code
-        if not config.SRC_EXTRA:
-            import pudb;pudb.set_trace()
-            append_odoo_src(config, config.files["odoo_docker_file"])
+        if include_src:
+            append_odoo_src(config, odoo_docker_file)
 
     else:
         # seems to use images from registry; no build section
@@ -1071,8 +1104,9 @@ def append_odoo_src(config, path):
     content = "ADD {src} /opt/src"
     path.write_text(path.read_text() + "\n" + content)
 
+
 def _complete_setting_name(ctx, param, incomplete):
-    lines = (current_dir / 'settings.txt').read_text().splitlines()
+    lines = (current_dir / "settings.txt").read_text().splitlines()
 
     params = []
     for line in lines:
@@ -1086,38 +1120,76 @@ def _complete_setting_name(ctx, param, incomplete):
         name = name.strip()
         doc = doc.strip()
         name = name.split("=")[0]
-        params.append({'name': name, 'doc': doc})
+        params.append({"name": name, "doc": doc})
 
-
-    res = set([x['name'].strip() for x in params])
+    res = set([x["name"].strip() for x in params])
     if incomplete:
         res = list(filter(lambda x: x.lower().startswith(incomplete.lower()), res))
     return sorted(res)
 
+
 @composer.command()
 @pass_config
 @click.pass_context
 @click.argument("name", required=False, shell_complete=_complete_setting_name)
 @click.argument("value", required=False, default="")
-def setting(ctx, config, name, value):
+@click.option("-R", "--no-reload", is_flag=True)
+def setting(ctx, config, name, value, no_reload):
     from .myconfigparser import MyConfigParser
+
     if not name:
         ctx.invoke(show_effective_settings)
         return
     configparser = MyConfigParser(config.files["settings"])
-    if '=' in name:
+    name = name.replace(":", "=")
+    if "=" in name:
         name, value = name.split("=", 1)
     if not value:
         for k in sorted(configparser.keys()):
-            if  name.lower() in k.lower():
+            if name.lower() in k.lower():
                 click.secho(f"{k}={configparser[k]}")
     else:
-        configparser = MyConfigParser(config.files['project_settings'])
-        configparser[name] = value
-        configparser.write()
-        click.secho(f"{name}={value}", fg='green')
+        update_setting(config, name, value)
+        click.secho(f"{name}={value}", fg="green")
+        if not no_reload:
+            ctx.invoke(do_reload)
+
+
+def _complete_services(ctx, param, incomplete):
+    content = _parse_yaml(__read_file(config.files["docker_compose"]))
+    services = content.get("services", {}).keys()
+
+    if incomplete:
+        services = list(
+            filter(lambda x: x.lower().startswith(incomplete.lower()), services)
+        )
+    return services
+
+
+@composer.command()
+@pass_config
+@click.pass_context
+@click.argument("name", required=True, shell_complete=_complete_services)
+@click.argument("value", required=False, default="")
+@click.option("--file", required=False, default="")
+@click.option("-R", "--no-reload", is_flag=True)
+def docker_service(ctx, config, name, value, no_reload, file):
+    from .myconfigparser import MyConfigParser
+    import yaml
+
+    if not value and not file:
+        content = _parse_yaml(__read_file(config.files["docker_compose"]))
+        service = content.get("services", {}).get(name)
+        click.secho(json.dumps(content, indent=4), fg="green")
+        return
+    if file:
+        value = __read_file(file)
+    value = _parse_yaml(value)
+    update_docker_service(config, name, value)
+    if not no_reload:
         ctx.invoke(do_reload)
 
+
 @composer.command()
 @pass_config
 @click.pass_context
@@ -1128,4 +1200,41 @@ def show_effective_settings(ctx, config):
     for k in sorted(config.keys()):
         click.echo(f"{k}={config[k]}")
 
+
+@composer.command()
+@click.argument("name", required=False)
+@click.argument("amount", required=False, type=int)
+@pass_config
+@click.pass_context
+def queuejob_channels(ctx, config, name, amount):
+    if name and amount is None:
+        abort(f"Please define the amount of workers in {name}")
+
+    from .myconfigparser import MyConfigParser
+
+    file = config.files["queuejob_channels_file"]
+    if not file.exists():
+        config = MyConfigParser(config.files["settings"])
+        content = config.get("ODOO_QUEUEJOBS_CHANNELS", "root:1")
+        file.write_text(content)
+    content = file.read_text()
+
+    channels = content.split(",")
+
+    if name:
+        channels = list(filter(lambda x: x.strip().split(":")[0] != name, channels))
+        channels.append(f"{name}:{amount}")
+        file.write_text(",".join(channels))
+        click.secho("Now restart the queuejob container", fg="yellow")
+    for channel in channels:
+        click.secho(f"{channel}", fg="green")
+
+
+def before_reload(config):
+    customs_dir = Path('.')
+    file = customs_dir / 'devscripts' / "before_reload.py"
+    if not file.exists():
+        return
+    subprocess.check_call(["python3", file])
+
 Commands.register(do_reload, "reload")
