@@ -264,15 +264,23 @@ def attach(ctx, config, machine):
 @click.option("--push", is_flag=True)
 @click.option("-p", "--plain", is_flag=True)
 @click.option("-s", "--include-source", is_flag=True)
+@click.option("-rm", "--remove", is_flag=True)
 @pass_config
 @click.pass_context
-def build(ctx, config, machines, pull, no_cache, push, plain, include_source):
+def build(ctx, config, machines, pull, no_cache, push, plain, include_source, remove):
+    import yaml
     ensure_project_name(config)
     if plain:
         os.environ["BUILDKIT_PROGRESS"] = "plain"
     from .lib_control_with_docker import build as lib_build
+    if not machines:
+        compose = yaml.safe_load(config.files["docker_compose"].read_text())
+        machines = []
+        for service in compose['services']:
+            if not compose['services'][service].get('build', {}).get('imgage'):
+                machines.append(service)
 
-    lib_build(ctx, config, machines, pull, no_cache, push, include_source)
+    lib_build(ctx, config, machines, pull, no_cache, push, include_source, remove)
 
 
 @docker.command()
@@ -285,6 +293,7 @@ def build(ctx, config, machines, pull, no_cache, push, plain, include_source):
 def debug(ctx, config, machine, ports, command, port):
     ensure_project_name(config)
     from .lib_control_with_docker import debug as lib_debug
+
     if port:
         ports = int(port)
 
@@ -294,13 +303,15 @@ def debug(ctx, config, machine, ports, command, port):
 @cli.command()
 @click.argument("machine", required=True)
 @click.argument("args", nargs=-1)
+@click.option("-d", "--detached", is_flag=True)
+@click.option("-n", "--name")
 @pass_config
 @click.pass_context
-def run(ctx, config, machine, args, **kwparams):
+def run(ctx, config, machine, detached, name, args, **kwparams):
     ensure_project_name(config)
     from .lib_control_with_docker import run as lib_run
 
-    lib_run(ctx, config, machine, args, **kwparams)
+    lib_run(ctx, config, machine, args, detached=detached, name=name, **kwparams)
 
 
 @cli.command()
@@ -386,7 +397,6 @@ def _get_project_volumes(config):
 @click.option("-f", "--filter")
 @pass_config
 def show_volumes(config, filter):
-    import yaml
     from tabulate import tabulate
     from .lib_control_with_docker import _get_volume_size
 
@@ -508,6 +518,37 @@ def transfer_volume_content(context, config, show_all, filter, no_backup):
     )
 
 
+@docker.command()
+@pass_config
+@click.pass_context
+def docker_sizes(context, config):
+    from .tools import __dc_out
+    from tabulate import tabulate
+
+    output = __dc_out(config, ["config"]).decode("utf-8")
+    # docker-compose config | grep "image:" | awk '{print $2}'
+    # docker images --format "{{.Repository}}:{{.Tag}} Size: {{.Size}}"
+    import yaml
+
+    image_names = list(
+        map(
+            lambda x: f"{config.project_name}_{x}",
+            yaml.safe_load(output)["services"].keys(),
+        )
+    )
+    out = subprocess.check_output(
+        ["docker", "images", "--format", "{{.Repository}}:{{.Tag}}\tSize: {{.Size}}"]
+    ).decode('utf8').splitlines()
+    sizes = {}
+    for line in out:
+        name, size = line.split(":", 1)
+        size = size.split("\t")[-1]
+        sizes[name] = size
+    recs = []
+    for name in sorted(image_names, key=lambda x: x[0]):
+        recs.append((name, sizes.get(name, "")))
+
+    click.echo(tabulate(recs, ["Image Name", "Size"]))
 
 
 Commands.register(run)
