@@ -125,7 +125,7 @@ def execute(config, machine, args):
     __dcexec(config, args)
 
 
-def do_kill(ctx, config, machines=[], brutal=False):
+def do_kill(ctx, config, machines=[], brutal=False, profile="auto"):
     """
     kills running machine
     safely shutdowns postgres and redis
@@ -147,15 +147,15 @@ def do_kill(ctx, config, machines=[], brutal=False):
                     safe_stop += [machine]
 
         if safe_stop:
-            __dc(config, ["stop", "-t 20"] + safe_stop)  # persist data
+            __dc(config, ["stop", "-t", "20"] + safe_stop, profile=profile)  # persist data
     if config.devmode:
-        __dc(config, ["kill"] + list(machines))
+        __dc(config, ["kill"] + list(machines), profile=profile)
     else:
-        __dc(config, ["stop", "-t 2"] + list(machines))
+        __dc(config, ["stop", "-t", "2"] + list(machines), profile=profile)
 
 
-def force_kill(ctx, config, machine):
-    do_kill(ctx, config, machine=machine, brutal=True)
+def force_kill(ctx, config, machine, profile="auto"):
+    do_kill(ctx, config, machine=machine, brutal=True, profile=profile)
 
 
 def wait_for_container_postgres(config):
@@ -173,7 +173,7 @@ def recreate(ctx, config, machines=[]):
     __dc(config, ["up", "--no-start", "--force-recreate"] + machines)
 
 
-def up(ctx, config, machines=[], daemon=False, remove_orphans=True):
+def up(ctx, config, machines=[], daemon=False, remove_orphans=True, profile="auto"):
     machines = list(machines)
 
     options = [
@@ -184,10 +184,13 @@ def up(ctx, config, machines=[], daemon=False, remove_orphans=True):
         options += ["-d"]
     if remove_orphans:
         options += ["--remove-orphans"]
+    dc_options = []
+    if profile:
+        dc_options += ["--profile", profile]
 
     if not machines and config.run_postgres and daemon and config.USE_DOCKER:
         _start_postgres_before(config)
-    __dc(config, ["up"] + options + machines)
+    __dc(config, dc_options + ["up"] + options + machines)
 
 
 def down(ctx, config, machines=[], volumes=False, remove_orphans=True):
@@ -244,13 +247,24 @@ def pull(ctx, config):
     __dc(config, ["pull"])
 
 
-def build(ctx, config, machines=[], pull=False, no_cache=False, push=False, include_source=False):
+def build(
+    ctx,
+    config,
+    machines=[],
+    pull=False,
+    no_cache=False,
+    push=False,
+    include_source=False,
+    remove=False,
+):
     """
     no parameter all machines, first parameter machine name and passes other params; e.g. ./odoo build asterisk --no-cache"
     """
     options = []
     if pull:
         options += ["--pull"]
+    if remove:
+        options += ["--force-rm"]
     if no_cache:
         options += ["--no-cache"]
         if "--pull" not in options:
@@ -327,7 +341,7 @@ def run(ctx, config, machine, args, **kwparams):
 def runbash(ctx, config, machine, args, **kwparams):
     _display_machine_tips(config, machine)
     bash = _get_bash_for_machine(machine)
-    cmd = ["run", "--rm", machine]
+    cmd = ["run", "--rm", "--entrypoint", "", machine]
     if args:
         cmd += args
     else:
