@@ -15,7 +15,10 @@ docker/w0v9nokwbqazajfri0u3k5cor
 better: zfs get -H -o value usedbychildren docker/volumes  mit -1 oder nicht
 
 """
+
 import inquirer
+import os
+import json
 from .tools import abort
 from operator import itemgetter
 import subprocess
@@ -30,7 +33,9 @@ from .cli import cli, pass_config
 from .lib_clickhelpers import AliasedGroup
 from pathlib import Path
 from .tools import abort
-from .tools import get_volume_fullpath
+from .tools import get_volume_fullpath, verbose
+
+MAX_I = 500
 
 HOWTO_PREPARE = """
 
@@ -46,6 +51,9 @@ systemctl start docker
 
 """
 
+CACHE_FILE_BASE = os.path.expanduser("~/.wodoo/zfs")
+
+
 try:
     zfs = search_env_path("zfs")
 except Exception:
@@ -66,10 +74,44 @@ def unify(text):
     return text
 
 
+def _is_zfs_path(path):
+    """
+    path e.g. tankdocker/volumes/postgres1
+    """
+    try:
+        subprocess.check_output(
+            ["sudo", zfs, "list", str(path)],
+            encoding="utf8",
+            stderr=subprocess.DEVNULL,  # ignore output of 'no datasets available'
+        ).strip().splitlines()[1:]
+        return True
+    except subprocess.CalledProcessError:
+        return False
+
+
 def _get_path(config):
     return get_volume_fullpath(__get_postgres_volume_name(config))
 
 
+def _cache_path(config):
+    assert config.project_name
+    return Path(CACHE_FILE_BASE) / config.project_name
+
+
+def _load_cache(config):
+    path = _cache_path(config)
+    if not path.exists():
+        return
+    for k, v in json.loads(path.read_text()).items():
+        _cache[k] = v
+
+
+def _save_cache(config):
+    path = _cache_path(config)
+    path.parent.mkdir(exist_ok=True, parents=True)
+    path.write_text(json.dumps(_cache))
+
+
 def _get_zfs_path(config):
     """
     takes the postgresname and translates:
@@ -78,16 +120,18 @@ def _get_zfs_path(config):
     Doesnt matter if pg1 is already a snapshot or not
 
     """
+    _load_cache(config)
     datasets = _get_all_mountpoints()
     viceversa = {v: k for k, v in datasets.items()}
     postgresname = __get_postgres_volume_name(config)
     firstmatch = [x for x in datasets.values() if x.endswith("/" + postgresname)]
     secondmatch = [x for x in datasets.values() if x.endswith("/volumes")]
+    _save_cache(config)
     if firstmatch:
         return viceversa[firstmatch[0]]
     if secondmatch:
         return viceversa[secondmatch[0]] + "/" + postgresname
-    raise Exception(f"Not found: {postgresname}")
+    raise Exception(f"Not found: {postgresname} in {','.join(datasets.keys())}")
 
 
 def _get_next_snapshotpath(config):
@@ -106,13 +150,16 @@ def _get_possible_snapshot_paths(config):
     :param path: root path
     """
     # TODO
-    all_zfs_folders = _get_all_zfs()
-    zfs_path = _get_zfs_path(config)
-    for folder in all_zfs_folders:
-        if folder == zfs_path:
-            yield folder
-        if folder.startswith(zfs_path + "."):
-            yield folder
+    # all_zfs_folders = _get_all_zfs()
+    base_path = _get_zfs_path(config)
+    for i in range(-1, MAX_I):
+        if i == -1:
+            if _is_zfs_path(base_path):
+                yield from _call_list_snapshots(base_path)
+        else:
+            path = f"{base_path}.{i}"
+            if _is_zfs_path(path):
+                yield from _call_list_snapshots(path)
 
 
 def __get_snapshots(config):
@@ -123,18 +170,24 @@ def __get_snapshots(config):
         abort(f"Path {path} is not a zfs.")
 
 
+def _call_list_snapshots(path):
+    res = (
+        subprocess.check_output(
+            ["sudo", zfs, "list", "-t", "snapshot", str(path)],
+            encoding="utf8",
+            stderr=subprocess.DEVNULL,  # ignore output of 'no datasets available'
+        )
+        .strip()
+        .splitlines()[1:]
+    )
+    res = list(map(lambda x: x.split()[0], res))
+    return res
+
+
 def _get_snapshots(config):
     def _get_snaps():
         for path in _get_possible_snapshot_paths(config):
-            for line in (
-                subprocess.check_output(
-                    ["sudo", zfs, "list", "-t", "snapshot", str(path)],
-                    encoding="utf8",
-                    stderr=subprocess.DEVNULL,  # ignore output of 'no datasets available'
-                )
-                .strip()
-                .splitlines()[1:]
-            ):
+            for line in _call_list_snapshots(path):
                 snapshotname = unify(line.split(" "))[0]
                 creation = unify(
                     subprocess.check_output(
@@ -167,6 +220,7 @@ def _get_all_zfs():
 def __is_zfs_fs(path_zfs):
     path_zfs = str(path_zfs)
     assert " " not in path_zfs
+    return _is_zfs_path(path_zfs)
     datasets = _get_all_zfs()
     return str(path_zfs) in datasets
 
@@ -207,6 +261,7 @@ def _turn_into_subvolume(config):
                 str(filename) + "/",
                 str(fullpath) + "/",
                 "-ar",
+                "--info=progress2",
             ]
         )
     finally:
@@ -215,7 +270,7 @@ def _turn_into_subvolume(config):
 
 def make_snapshot(ctx, config, name):
     zfs = search_env_path("zfs")
-    __dc(config, ["stop", "-t 1"] + ["postgres"])
+    __dc(config, ["stop", "-t", "1"] + ["postgres"])
     _turn_into_subvolume(config)
     snapshots = list(_get_snapshots(config))
     snapshot = list(filter(lambda x: x["name"] == name, snapshots))
@@ -282,20 +337,25 @@ def restore(config, name):
     except ValueError:
         index = -1
 
-    __dc(config, ["stop", "-t 1"] + ["postgres"])
+    __dc(config, ["stop", "-t", "1"] + ["postgres"])
     if index == 0:
         # restore last one is easy in the volumefolder it self; happiest case
         subprocess.check_call(["sudo", zfs, "rollback", snapshot["fullpath"]])
     else:
         full_next_path = _get_next_snapshotpath(config)
         _try_umount(config)
-        subprocess.check_call(["sudo", zfs, "rename", zfs_full_path, full_next_path])
+        if _is_zfs_path(zfs_full_path):
+            subprocess.check_call(
+                ["sudo", zfs, "rename", zfs_full_path, full_next_path]
+            )
+        snap_name = snapshot["fullpath"].split("@")[-1]
+        snapshot_path = _get_zfs_path_for_snap_name(config, snap_name)
         subprocess.check_call(
             [
                 "sudo",
                 zfs,
                 "clone",
-                snapshot["fullpath"],
+                snapshot_path,
                 zfs_full_path,
             ]
         )
@@ -303,6 +363,13 @@ def restore(config, name):
     __dc(config, ["up", "-d"] + ["postgres"])
 
 
+def _get_zfs_path_for_snap_name(config, snap_name):
+    for path in _get_possible_snapshot_paths(config):
+        if path.split("@")[-1] == snap_name:
+            return path
+    abort(f"Could not find snapshot with name {snap_name}")
+
+
 def remove(config, snapshot):
     zfs = search_env_path("zfs")
     snapshots = __get_snapshots(config)
@@ -322,7 +389,13 @@ def remove_volume(config):
     zfs = search_env_path("zfs")
     umount = search_env_path("umount")
     for path in _get_possible_snapshot_paths(config):
-        subprocess.check_call(["sudo", zfs, "set", "canmount=noauto", path])
+        try:
+            subprocess.check_call(["sudo", zfs, "set", "canmount=noauto", path])
+        except:
+            click.secho(
+                "Failed to execute canmount=noauto, but perhaps not a problem. Trying to continue.",
+                fg="yellow",
+            )
         try:
             subprocess.check_call(["sudo", umount, path])
         except:
@@ -332,21 +405,25 @@ def remove_volume(config):
     clear_all(config)
 
 
-def _get_pool_mountpoint(poolname):
-    mountpoint = subprocess.check_output(
-        ["sudo", zfs, "get", "mountpoint", "-H", "-o", "value", poolname],
-        encoding="utf8",
-    ).strip()
-    return Path(mountpoint)
+def translate_poolPath_to_fullPath(zfs_path):
+    zfs_path = Path(zfs_path)
+    removed = []
+    while len(zfs_path.parts) > 1:
+        mountpoint = None
+        try:
+            mountpoint = subprocess.check_output(
+                ["sudo", zfs, "get", "mountpoint", "-H", "-o", "value", zfs_path],
+                encoding="utf8",
+            ).strip()
+        except:
+            mountoint = None
+        if mountpoint != "-":
+            break
+        removed.insert(0, zfs_path.parts[-1])
+        zfs_path = Path("/".join(zfs_path.parts[:-1]))
+    mountpoint = Path(mountpoint) / "/".join(removed) if mountpoint else None
 
-
-def translate_poolPath_to_fullPath(path):
-    # TODO
-    path = Path(path)
-    pool = path.parts[0]
-    poolpath = _get_pool_mountpoint(pool)
-    path = poolpath / path.relative_to(pool)
-    return path
+    return Path(mountpoint) if mountpoint else None
 
 
 def clear_all(config):
@@ -354,7 +431,7 @@ def clear_all(config):
     zfs_full_path = _get_zfs_path(config)
     _try_umount(config)
     diskpath = translate_poolPath_to_fullPath(zfs_full_path)
-    if __is_zfs_fs(diskpath):
+    if diskpath and __is_zfs_fs(diskpath):
         subprocess.check_call(["sudo", zfs, "destroy", "-r", zfs_full_path])
 
 
