@@ -19,6 +19,8 @@ import click
 from wodoo.robo_helpers import _get_required_odoo_modules_from_robot_file
 from .tools import is_git_clean
 from gimera.repo import Repo
+from .tools import _make_sure_module_is_installed
+from .tools import autocleanpaper
 from .tools import get_hash
 from .tools import get_directory_hash
 from .tools import sync_folder
@@ -36,13 +38,34 @@ from .tools import measure_time, abort
 from .tools import _update_setting
 from .tools import _get_setting
 from .tools import get_git_hash
+from .tools import start_postgres_if_local
 from .module_tools import _determine_affected_modules_for_ir_field_and_related
 from pathlib import Path
+from functools import partial
 
 DTF = "%Y-%m-%d %H:%M:%S"
 KEY_SHA_REVISION = "sha.revision"
 
 
+def get_all_modules_installed_by_manifest(config):
+    from .module_tools import Modules, DBModules, Module
+
+    manifest_modules = Modules().get_all_modules_installed_by_manifest()
+    remove_some_modules(config, manifest_modules)
+    return manifest_modules
+
+
+def remove_some_modules(config, modules):
+    from .odoo_config import MANIFEST
+
+    devmode_uninstall = MANIFEST().get("devmode_uninstall", [])
+    if config.devmode:
+        for mod in devmode_uninstall:
+            if mod in modules:
+                modules.remove(mod)
+            del mod
+
+
 class UpdateException(Exception):
     pass
 
@@ -68,13 +91,16 @@ def abort_upgrade(config):
     DBModules.abort_upgrade()
 
 
-def _get_default_modules_to_update():
+def _get_default_modules_to_update(config):
     from .module_tools import Modules, DBModules
+    from .odoo_config import MANIFEST
 
     mods = Modules()
     module = mods.get_customs_modules("to_update")
     module += DBModules.get_uninstalled_modules_where_others_depend_on()
     module += DBModules.get_outdated_installed_modules(mods)
+    remove_some_modules(config, module)
+
     return module
 
 
@@ -91,6 +117,7 @@ def update_module_file(module):
 @pass_config
 @click.pass_context
 def run_tests(ctx, config):
+    start_postgres_if_local(ctx, config)
     started = datetime.now()
     if not config.devmode and not config.force:
         click.secho(
@@ -114,11 +141,13 @@ def run_tests(ctx, config):
         click.secho("No test files found!")
         return
 
-    if config.force:
-        Commands.invoke(ctx, "wait_for_container_postgres", missing_ok=True)
-        Commands.invoke(ctx, "reset-db")
-        Commands.invoke(ctx, "update", "", tests=False, no_dangling_check=True)
+    def reset_db():
+        if config.force:
+            Commands.invoke(ctx, "wait_for_container_postgres", missing_ok=True)
+            Commands.invoke(ctx, "reset-db")
+            Commands.invoke(ctx, "update", "", tests=False, no_dangling_check=True)
 
+    reset_db()
     from .module_tools import Module
     from .odoo_config import customs_dir
 
@@ -132,27 +161,56 @@ def run_tests(ctx, config):
         testfiles = [x for x in testfiles if x.name.startswith("test_")]
 
         # identify test files and run them, otherwise tests of dependent modules are run
+        ran_tests = []
         for file in sorted(testfiles):
-            mfpath = module.manifest_path.parent
-            file = mfpath.relative_to(customs_dir()) / file
+            file = module.path / file
+            ran_tests.append(file)
+
+            print(f"So far tests being run:")
+            for i, txtfile in enumerate(ran_tests, 1):
+                print(f"{i}: {txtfile}")
+
             if config.use_docker:
-                params = ["odoo", "/odoolib/unit_test.py", f"{file}"]
-                click.secho(f"Running test: {file}", fg="yellow", bold=True)
-                res = __dcrun(
-                    config,
-                    params + ["--log-level=error", "--not-interactive"],
-                    returncode=True,
-                )
+
+                def run_test(file):
+                    params = ["odoo", "/odoolib/unit_test.py", file]
+                    click.secho(f"Running test: {file}", fg="yellow", bold=True)
+                    res = __dcrun(
+                        config,
+                        params + ["--log-level=error", "--not-interactive"],
+                        returncode=True,
+                    )
+                    return res
+
+                res = run_test(file)
                 if res:
-                    failed.append(file)
-                    click.secho(
-                        f"Failed, running again with debug on: {file}",
-                        fg="red",
-                        bold=True,
-                    )
-                    res = __cmd_interactive(
-                        config, *(["run", "--rm"] + params + ["--log-level=debug"])
+                    print(
+                        f"Test {file} failed on first attempt. Resetting db and trying once more."
                     )
+                    reset_db()
+                    res = run_test(file)
+                    if res:
+                        failed.append(file)
+                        click.secho(
+                            f"Failed, running again with debug on: {file}",
+                            fg="red",
+                            bold=True,
+                        )
+                        res = __cmd_interactive(
+                            config,
+                            *(
+                                [
+                                    "run",
+                                    "--rm",
+                                    "odoo",
+                                    "/odoolib/unit_test.py",
+                                    file,
+                                    "--log-level=debug",
+                                ]
+                            ),
+                        )
+                    else:
+                        success.append(file)
                 else:
                     success.append(file)
 
@@ -161,7 +219,11 @@ def run_tests(ctx, config):
 
     # in force-mode shut down
     if config.force:
-        Commands.invoke(ctx, "down", volumes=True)
+        for _ in range(3):
+            try:
+                Commands.invoke(ctx, "down", volumes=True)
+            except:
+                time.sleep(3)
 
     if failed:
         click.secho("Tests failed: ", fg="red")
@@ -415,6 +477,11 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
     is_flag=False,
     help="Extracts modules changed since this git sha and updates them",
 )
+@click.option(
+    "--stdout",
+    is_flag=False,
+    help="directs stdout to given file",
+)
 @pass_config
 @click.pass_context
 def update(
@@ -442,6 +509,7 @@ def update(
     log=False,
     recover_view_error=False,
     no_outdated_modules=False,
+    stdout=False,
 ):
     """
     Just custom modules are updated, never the base modules (e.g. prohibits adding old stock-locations)
@@ -499,9 +567,7 @@ def update(
     if test_tags and default_test_tags:
         abort("Conflict: parameter test-tags and default-test-tags")
 
-    if config.run_postgres:
-        Commands.invoke(ctx, "up", machines=["postgres"], daemon=True)
-        Commands.invoke(ctx, "wait_for_container_postgres", missing_ok=True)
+    start_postgres_if_local(ctx, config)
 
     def _perform_install(module):
         if since_git_sha and module:
@@ -516,7 +582,7 @@ def update(
             module = _parse_modules(module)
 
         if not module and not since_git_sha:
-            module = _get_default_modules_to_update()
+            module = _get_default_modules_to_update(config)
 
         def _get_outdated_modules():
             return list(
@@ -588,12 +654,21 @@ def update(
                 if log:
                     params += [f"--log={log}"]
                 params += ["--config-file=" + config_file]
-                rc = list(_exec_update(config, params, non_interactive=non_interactive))
+                rc = list(
+                    _exec_update(
+                        config,
+                        params,
+                        non_interactive=non_interactive if not stdout else True,
+                    )
+                )
                 if len(rc) == 1:
                     rc = rc[0]
                 else:
                     rc, output = rc
 
+                if stdout:
+                    Path(stdout).write_text(output)
+
                 if rc:
                     if recover_view_error:
                         try:
@@ -656,6 +731,9 @@ def update(
     if not uninstall:
         _perform_install(module)
 
+    manifest = MANIFEST()
+    _uninstall_devmode_modules(ctx, config, manifest)
+
     all_modules = (
         not param_module
         or len(param_module) == 1
@@ -663,7 +741,7 @@ def update(
     )
 
     if uninstall or all_modules:
-        _uninstall_marked_modules(ctx, config, MANIFEST().get("uninstall", []))
+        _uninstall_marked_modules(ctx, config, manifest.get("uninstall", []))
 
     # check danglings
     if not no_dangling_check and all_modules:
@@ -679,6 +757,12 @@ def update(
     click.secho(f"Update done at {date} - duration {duration}s", fg="yellow")
 
 
+def _uninstall_devmode_modules(ctx, config, manifest):
+    if config.devmode and manifest.get("devmode_uninstall", []):
+        for mod in manifest["devmode_uninstall"]:
+            _uninstall_marked_modules(ctx, config, manifest["devmode_uninstall"])
+
+
 def _set_sha(config):
     conn = config.get_odoo_conn()
     try:
@@ -743,7 +827,9 @@ def _do_check_install_state(ctx, config, module, all_modules, no_dangling_check)
             missing_as_error=True,
         )
     else:
-        missing = list(DBModules.check_if_all_modules_from_install_are_installed())
+        missing = DBModules.check_if_all_modules_from_install_are_installed(
+            partial(remove_some_modules, config)
+        )
         problem_missing = set()
         for module in module:
             if module in missing:
@@ -808,6 +894,7 @@ def _uninstall_marked_modules(ctx, config, modules):
     Checks for file "uninstall" in customs root and sets modules to uninstalled.
     """
     from .module_tools import Modules, DBModules, Module
+    from .module_tools import NotInAddonsPath
 
     assert not isinstance(modules, str)
 
@@ -823,9 +910,21 @@ def _uninstall_marked_modules(ctx, config, modules):
 
     if config.use_docker:
         from .lib_control_with_docker import shell as lib_shell
+    manifest_modules = get_all_modules_installed_by_manifest(config)
 
     uninstalled = False
     for module in modules:
+        try:
+            objmod = Module.get_by_name(module)
+            for desc in objmod.descendants:
+                if desc in manifest_modules:
+                    if not Module.get_by_name(desc).manifest_dict.get("auto_install"):
+                        abort(
+                            f"{objmod.name} has {desc.name} as descendant which is still in the install section"
+                        )
+        except NotInAddonsPath:
+            pass
+
         click.secho(f"Uninstall {module}", fg="red")
         lib_shell(
             config,
@@ -868,7 +967,7 @@ def update_i18n(ctx, config, module, no_restart):
     )  # '1,2 3' --> ['1', '2', '3']
 
     if not module:
-        module = _get_default_modules_to_update()
+        module = _get_default_modules_to_update(config)
 
     try:
         params = [",".join(module)]
@@ -900,7 +999,9 @@ def show_install_state(config, suppress_error=False, missing_as_error=False):
         click.echo("{}: {}".format(row[0], row[1]))
 
     # get modules, that are not installed:
-    missing = list(DBModules.check_if_all_modules_from_install_are_installed())
+    missing = DBModules.check_if_all_modules_from_install_are_installed(
+        partial(remove_some_modules, config)
+    )
     for missing_item in missing:
         click.secho((f"Module {missing_item} not installed!"), fg="red")
 
@@ -932,7 +1033,7 @@ def show_conflicting_modules():
     get_odoo_addons_paths()
 
 
-def _exec_update(config, params, non_interactive=False):
+def _exec_update(config, params, non_interactive=False, stdout=False):
     params = ["odoo_update", "/update_modules.py"] + params
     if not non_interactive:
         yield __cmd_interactive(
@@ -954,180 +1055,6 @@ def _exec_update(config, params, non_interactive=False):
             yield ex.returncode
 
 
-def _get_available_robottests(ctx, param, incomplete):
-    from .robo_helpers import _get_all_robottest_files
-    from .odoo_config import customs_dir
-
-    path = customs_dir() or Path(os.getcwd())
-    path = path / (Path(os.getcwd()).relative_to(path))
-    testfiles = list(map(str, _get_all_robottest_files(path))) or []
-    if incomplete:
-        if "/" in incomplete:
-            testfiles = list(filter(lambda x: str(x).startswith(incomplete), testfiles))
-        else:
-            testfiles = list(filter(lambda x: incomplete in x, testfiles))
-    return sorted(testfiles)
-
-
-@odoo_module.command()
-@click.argument("file", required=False, shell_complete=_get_available_robottests)
-@click.option("-u", "--user", default="admin")
-@click.option("-a", "--all", is_flag=True)
-@click.option("-n", "--test_name", is_flag=False)
-@click.option(
-    "-p", "--param", multiple=True, help="e.g. --param key1=value1 --param key2=value2"
-)
-@click.option("--parallel", default=1, help="Parallel runs of robots.")
-@click.option(
-    "--keep-token-dir",
-    is_flag=True,
-    help="If set, then the intermediate run directory is kept. Helps to separate test runs of same robot file safely.",
-)
-@click.option(
-    "-t",
-    "--tags",
-    is_flag=False,
-    help=(
-        "Tags can be comined with AND OR or just comma separated; "
-        "may include wilcards and some regex expressions"
-    ),
-)
-@click.option(
-    "-j",
-    "--output-json",
-    is_flag=True,
-    help=("If set, then a json is printed to console, with detailed informations"),
-)
-@click.option(
-    "--results-file", help="concrete filename where the results.json is stored"
-)
-@pass_config
-@click.pass_context
-def robotest(
-    ctx,
-    config,
-    file,
-    user,
-    all,
-    tags,
-    test_name,
-    param,
-    parallel,
-    output_json,
-    keep_token_dir,
-    results_file,
-):
-    PARAM = param
-    del param
-    started = arrow.utcnow()
-
-    from pathlib import Path
-    from .odoo_config import customs_dir
-    from .module_tools import DBModules
-
-    if not config.devmode and not config.force:
-        click.secho(
-            ("Devmode required to run unit tests. Database will be destroyed."),
-            fg="red",
-        )
-        sys.exit(-1)
-
-    from .robo_helpers import _select_robot_filename
-
-    filenames = _select_robot_filename(file, run_all=all)
-    del file
-
-    if not filenames:
-        return
-
-    click.secho("\n".join(map(str, filenames)), fg="green", bold=True)
-
-    from .robo_helpers import get_odoo_modules
-
-    os.chdir(customs_dir())
-    odoo_modules = set(get_odoo_modules(config.verbose, filenames, customs_dir()))
-    odoo_modules = list(odoo_modules | set(["web_selenium", "robot_utils"]))
-
-    if odoo_modules:
-
-        def not_installed(module):
-            data = DBModules.get_meta_data(module)
-            if not data:
-                abort(f"Could not get state for {module}")
-            return data["state"] == "uninstalled"
-
-        modules_to_install = list(filter(not_installed, odoo_modules))
-        if modules_to_install:
-            click.secho(
-                (
-                    "Installing required modules for robot tests: "
-                    f"{','.join(modules_to_install)}"
-                ),
-                fg="yellow",
-            )
-            Commands.invoke(
-                ctx, "update", module=modules_to_install, no_dangling_check=True
-            )
-
-    pwd = config.DEFAULT_DEV_PASSWORD
-    if pwd == "True" or pwd is True:
-        pwd = "1"
-
-    def params():
-        params = {
-            "url": "http://proxy",
-            "user": user,
-            "dbname": config.DBNAME,
-            "password": pwd,
-            "selenium_timeout": 20,  # selenium timeout,
-            "parallel": parallel,
-        }
-        if test_name:
-            params["test_name"] = test_name
-        if tags:
-            params["tags"] = tags
-
-        for param in PARAM:
-            k, v = param.split("=")
-            params[k] = v
-            del param
-
-        return params
-
-    token = arrow.get().strftime("%Y-%m-%d_%H%M%S_") + str(uuid.uuid4())
-    data = json.dumps(
-        {
-            "test_files": list(map(str, filenames)),
-            "token": token,
-            "results_file": results_file or "",
-            "params": params(),
-        }
-    )
-    data = base64.b64encode(data.encode("utf-8"))
-
-    params = [
-        "robot",
-    ]
-
-    workingdir = customs_dir() / (Path(os.getcwd()).relative_to(customs_dir()))
-    os.chdir(workingdir)
-
-    __dcrun(config, params, pass_stdin=data.decode("utf-8"), interactive=True)
-
-    output_path = config.HOST_RUN_DIR / "odoo_outdir" / "robot_output"
-    from .robo_helpers import _eval_robot_output
-
-    _eval_robot_output(
-        config,
-        output_path,
-        started,
-        output_json,
-        token,
-        rm_tokendir=not keep_token_dir,
-        results_file=results_file,
-    )
-
-
 def _get_unittests_from_module(module_name):
     from .module_tools import Module
     from .odoo_config import MANIFEST_FILE
@@ -1147,10 +1074,7 @@ def _get_unittests_from_modules(module_names):
 
 
 def _get_all_unittest_files(config):
-    from .odoo_config import MANIFEST
-    from .module_tools import Modules
-
-    modules = Modules().get_all_modules_installed_by_manifest()
+    modules = get_all_modules_installed_by_manifest(config)
     return _get_unittests_from_modules(modules)
 
 
@@ -1223,18 +1147,13 @@ def unittest(
         return
 
     for todoitem in todo:
-        click.secho(str(todoitem), fg="green", bold=True)
-
-    def filepath_to_container(filepath):
-        return Path("/opt/src/") / filepath
-
-    container_files = list(map(filepath_to_container, todo))
+        click.secho(str(todoitem), fg="blue", bold=True)
 
     interactive = True  # means pudb trace turned on
     params = [
         "odoo",
         "/odoolib/unit_test.py",
-        f'{",".join(map(str, container_files))}',
+        f'{",".join(map(str, todo))}',
     ]
     if wait_for_remote:
         remote_debug = True
@@ -1292,7 +1211,7 @@ def unittest(
 @pass_config
 @click.pass_context
 def generate_update_command(ctx, config):
-    modules = _get_default_modules_to_update()
+    modules = _get_default_modules_to_update(config)
     click.secho(f"-u {','.join(modules)}")
 
 
@@ -1394,11 +1313,30 @@ def _get_directory_hash(path):
 
 
 @odoo_module.command()
-@click.argument("module", required=True)
-@click.option("-N", "--no-cache", is_flag=True)
+@click.argument("module", required=True, shell_complete=_get_available_modules)
+@click.option("-f", "--following", is_flag=True)
+@click.option("-c", "--customs", is_flag=True)
 @pass_config
 @click.pass_context
-def list_deps(ctx, config, module, no_cache):
+def list_descendants(ctx, config, module, customs, following):
+    from .module_tools import Modules, DBModules, Module
+    from .module_tools import NotInAddonsPath
+    from .odoo_config import customs_dir
+    from .consts import FILE_DIRHASHES
+
+    mod = Module.get_by_name(module)
+    res = mod.descendants
+    for mod in res:
+        click.secho(f"descedant: {mod}", fg="green")
+
+
+@odoo_module.command()
+@click.argument("module", required=True, shell_complete=_get_available_modules)
+@click.option("-N", "--no-cache", is_flag=True)
+@click.option("-c", "--customs", is_flag=True)
+@pass_config
+@click.pass_context
+def list_deps(ctx, config, module, no_cache, customs):
     import arrow
 
     started = arrow.get()
@@ -1484,9 +1422,24 @@ def list_deps(ctx, config, module, no_cache):
             result[module.name] = data
 
     click.secho("---")
+    if customs:
+        result = _filter_customs(result)
     click.secho(json.dumps(result, indent=4))
 
 
+def _filter_customs(modules):
+    modules2 = {}
+    from .module_tools import DBModules, Module
+
+    def _filter(module_name):
+        mod = Module.get_by_name(module_name)
+        return mod.is_customs
+
+    for key in ["modules", "auto_install"]:
+        modules[key] = list(filter(_filter, modules[key]))
+    return modules
+
+
 @odoo_module.command()
 def migrate():
     click.secho(
@@ -1507,10 +1460,7 @@ def migrate():
 @pass_config
 @click.pass_context
 def list_modules(ctx, config):
-    from .module_tools import Modules, DBModules
-
-    mods = Modules()
-    modules = list(sorted(mods.get_all_modules_installed_by_manifest()))
+    modules = list(sorted(get_all_modules_installed_by_manifest(config)))
     print("---")
     for m in modules:
         print(m)
@@ -1520,7 +1470,7 @@ def list_modules(ctx, config):
 @pass_config
 @click.pass_context
 def list_outdated_modules(ctx, config):
-    modules = _get_default_modules_to_update()
+    modules = _get_default_modules_to_update(config)
 
     def _get_outdated_modules(module):
         return list(
@@ -1598,7 +1548,7 @@ def list_installed_modules(config, fix_not_in_manifest, only_customs):
             except Module.IsNot:
                 click.secho(f"Ignoring {module} - not found in source", fg="yellow")
                 continue
-            if any(x in parts for x in ["odoo", "enterprise", "themes"]):
+            if not mod.is_customs:
                 continue
         try:
             click.secho(f"{module}: {mod.path}", fg="green")
@@ -1631,6 +1581,7 @@ def zip(config, ctx, module):
 
     try:
         mod = Module.get_by_name(module)
+        module_path = Path(os.getcwd()) / mod.path
     except:
         module_path = Path(os.getcwd()) / module
 
@@ -1640,11 +1591,35 @@ def zip(config, ctx, module):
     zipfile = Path(os.getcwd()) / f"{module}.zip"
     if zipfile.exists():
         zipfile.unlink()
-    subprocess.check_call(
-        ["/usr/bin/zip", "-r", zipfile.name, "."], cwd=module_path
-    )
+    subprocess.check_call(["/usr/bin/zip", "-r", zipfile.name, "."], cwd=module_path)
+    shutil.move(Path(module_path / zipfile.name), zipfile)
+    click.secho(f"Created zipfile: {zipfile}", fg="green")
 
 
+@odoo_module.command()
+@click.argument("module", required=True)
+@click.argument("repourl", required=True)
+@click.option("--update", is_flag=True)
+@click.pass_context
+@pass_config
+def download_module(config, ctx, module, repourl, update):
+    _make_sure_module_is_installed(ctx, config, module, repourl, update)
+    click.secho(f"Successfully downloaded and installed {module}", fg="green")
+
+
+@odoo_module.command()
+@click.pass_context
+@pass_config
+def scan_addons_paths(config, ctx):
+    from .odoo_config import customs_dir
+
+    res = set()
+    root = customs_dir()
+    for path in root.rglob("__manifest__.py"):
+        res.add(path.parent.parent.relative_to(root))
+    for item in sorted(res):
+        click.secho(str(item))
+
 
 Commands.register(update)
 Commands.register(show_install_state)
