@@ -28,6 +28,8 @@ from .tools import _get_customs_root
 from .tools import _get_xml_id
 from .tools import pretty_xml
 from .tools import _execute_sql
+from .tools import exe
+from .tools import odoorpc
 
 
 ADDONS_OCA = "addons_OCA"
@@ -47,7 +49,7 @@ def _find_duplicate_modules():
     _identify_duplicate_modules(all_modules)
 
 
-def _apply_gimera_if_required(ctx, path, content, force_do=False):
+def _apply_gimera_if_required(ctx, path, content, force_do=False, no_fetch=None):
     from gimera.gimera import apply as gimera
 
     with cwd(path):
@@ -55,7 +57,12 @@ def _apply_gimera_if_required(ctx, path, content, force_do=False):
             repo_path = path / repo["path"]
             if repo["type"] == "submodule" or force_do or not repo_path.exists():
                 ctx.invoke(
-                    gimera, repos=[repo["path"]], recursive=True, no_auto_commit=True
+                    gimera,
+                    repos=[repo["path"]],
+                    recursive=True,
+                    no_patches=True,
+                    non_interactive=True,
+                    no_auto_commit=True,
                 )
                 changed = True
         else:
@@ -75,11 +82,12 @@ def _apply_gimera_if_required(ctx, path, content, force_do=False):
 
 @src.command()
 @click.pass_context
-def apply_gimera_if_required(ctx):
+@click.option("--no-fetch", is_flag=True)
+def apply_gimera_if_required(ctx, no_fetch):
     path = customs_dir()
     gimera_file = path / "gimera.yml"
     repos = yaml.safe_load(gimera_file.read_text())
-    _apply_gimera_if_required(ctx, path, repos)
+    _apply_gimera_if_required(ctx, path, repos, not no_fetch)
 
 
 @src.command()
@@ -450,11 +458,38 @@ def delete_modules_not_in_manifest(config, ctx, dry_run):
                 shutil.rmtree(root / mod.path)
 
 
+@src.command()
+@click.argument("path", required=True)
+@click.pass_context
+@pass_config
+def restore_view(config, ctx, path):
+    odoo = odoorpc(config)
+    langs = [x.code for x in odoo.env['res.lang'].browse(odoo.env["res.lang"].search([]))]
+    path = Path(path)
+    content = path.read_text()
+    xmlid = Path(path.name.split(".xmlid.")[1]).stem
+    lang = xmlid.split(".lang.")[-1].split(".")[0]
+    assert lang in langs
+    xmlid = xmlid.split(".lang.")[0]
+    module, name = xmlid.split(".")[0], ".".join(xmlid.split(".")[1:])
+    data = odoo.env["ir.model.data"].browse(
+        odoo.env["ir.model.data"].search(
+            [("module", "=", module), ("name", "=", name), ("model", "=", "ir.ui.view")]
+        )
+    )
+    view_id = data[0].res_id
+    odoo.env["ir.ui.view"].browse(view_id).write(
+        {"arch_db": content}, context={"lang": lang, 'mickey': 'mouse'}
+    )
+    click.secho(f"View [{view_id}] {module}.{name} updated successfully for language {lang}", fg="green")
+
+
 @src.command()
 @click.pass_context
 @pass_config
 def grab_views(config, ctx):
     root = customs_dir() / "src" / "views"
+    odoo = odoorpc(config)
 
     sql = f"select id, name, arch_db, model from ir_ui_view"
     conn = config.get_odoo_conn()
@@ -467,35 +502,37 @@ def grab_views(config, ctx):
     threads = []
     stats = {"views": 0}
     q = Queue()
+    langs = [x.code for x in odoo.env['res.lang'].browse(odoo.env["res.lang"].search([]))]
 
     def check_view():
         while not q.empty():
             count, view = q.get()
             try:
+                for lang in langs:
 
-                def prog(c):
-                    p = round(count / len(rows) * 100, 1)
-                    click.secho(f"...threading progress {p}%", fg="yellow")
+                    def prog(c):
+                        p = round(count / len(rows) * 100, 1)
+                        click.secho(f"...threading progress {p}%", fg="yellow")
 
-                arch = view[2]
-                xmlid = _get_xml_id(config, "ir.ui.view", view[0])
-                model = view[3] or "no-model"
-                name = view[1]
-                id = view[0]
-                if current_version() < 16:
-                    arch = {"en_US": arch}
+                    xml = odoo.env['ir.ui.view'].read([view[0]], ["arch_db"], context={"lang": lang})[0]['arch_db']
+                    xmlid = _get_xml_id(config, "ir.ui.view", view[0])
+                    model = view[3] or "no-model"
+                    name = view[1]
+                    id = view[0]
 
-                for key in arch:
                     if xmlid:
-                        filepath = root / f"{model}.xmlid.{xmlid}.xml"
+                        filepath = root / f"{model}.xmlid.{xmlid}.lang.{lang}.xml"
                     else:
                         filepath = (
-                            root / "by_name" / f"{model}.byname.{name}.{key}.{id}.xml"
+                            root
+                            / "by_name"
+                            / f"{model}.byname.{name}.{id}.lang.{lang}.xml"
                         )
                     filepath.parent.mkdir(exist_ok=True, parents=True)
 
-                    path = filepath.parent / (filepath.stem + f".{key}.xml")
-                    xml = arch[key].encode("utf8")
+                    path = filepath.parent / (filepath.stem + ".xml")
+
+                    xml = xml.encode("utf8")
                     path.write_bytes(pretty_xml(xml))
                     if path in all_files:
                         all_files.remove(path)
@@ -509,7 +546,7 @@ def grab_views(config, ctx):
     for count, view in enumerate(rows):
         q.put((count, view))
 
-    for i in range(10):
+    for i in range(15):
         t = threading.Thread(target=check_view)
         t.daemon = True
         t.start()
@@ -689,3 +726,110 @@ def grab_models(config, ctx):
         t.start()
         threads.append(t)
     [t.join() for t in threads]
+
+
+@src.command()
+@click.argument(
+    "module", nargs=-1, shell_complete=_get_available_oca_modules, required=False
+)
+@click.pass_context
+@pass_config
+def convert_odoo17_attrs(config, ctx, module):
+    from .module_tools import Modules, DBModules, Module
+    from .lib_src_replace_attrs import odoo17attrs
+
+    modules = Modules()
+    all_modules = modules.get_all_modules_installed_by_manifest()
+    for m in all_modules:
+        if (module and m in module) or not module:
+            m = Module.get_by_name(m)
+            if m.is_customs or not module:
+                odoo17attrs(customs_dir() / m.path)
+
+
+@src.command()
+@click.argument("sha", required=True)
+@click.pass_context
+@pass_config
+def analyze(config, ctx, sha):
+    from .module_tools import Modules, DBModules, Module
+    from .lib_src_replace_attrs import odoo17attrs
+
+    result = {
+        "manifests_changed": set(),
+        "updated_modules": set(),
+        "changed_xml_files": set(),
+    }
+
+    pychanges = subprocess.check_output(
+        ["git", "log", "-p", f"{sha}..HEAD", "--", "*.py"],
+        encoding="utf8",
+        stderr=subprocess.DEVNULL,
+    )
+    filesoutput = subprocess.check_output(
+        ["git", "log", "--name-only", "--pretty=format:", f"{sha}..HEAD"],
+        encoding="utf8",
+    ).splitlines()
+    files = list(
+        set(
+            filter(
+                bool,
+                map(lambda x: x.strip(), filesoutput),
+            )
+        )
+    )
+
+    def _get_module_name(x):
+        x = x.split("/")[-2].strip()
+        if x == "...":
+            x = ""
+        try:
+            m = Module.get_by_name(x)
+        except Exception:
+            return ""
+        else:
+            return str(m.name)
+
+    result["changed_xml_files"] |= set(
+        filter(
+            bool,
+            [_get_module_name(x) for x in files if ".xml" in x],
+        )
+    )
+    result["manifests_changed"] |= set(
+        filter(
+            bool,
+            [_get_module_name(x) for x in files if "__manifest__.py" in x],
+        )
+    )
+
+    for module, line in splitdiff(pychanges):
+        result["updated_modules"].add(str(module.name))
+
+    result["updated_modules"] = list(sorted(result["updated_modules"]))
+    result["changed_xml_files"] = list(sorted(result["changed_xml_files"]))
+    result["manifests_changed"] = list(sorted(result["manifests_changed"]))
+    print("----------------")
+    print(json.dumps(result))
+
+
+def splitdiff(diffoutput):
+    module = None
+    from .module_tools import Module
+
+    remember = set()
+    for line in diffoutput.splitlines():
+        if line.startswith("diff --git"):
+            filepath = line.split("b/")[0].split("a/")[1].strip()
+            for part in Path(filepath).parts[:-1]:
+                if part in remember:
+                    continue
+                try:
+                    module = Module.get_by_name(part)
+                except Exception:
+                    remember.add(part)
+                else:
+                    break
+
+        if module:
+            yield module, line
