@@ -1,4 +1,6 @@
 import json
+import arrow
+import time
 from datetime import datetime
 from pathlib import Path
 
@@ -9,6 +11,7 @@ from .cli import Commands, cli, pass_config
 from .lib_clickhelpers import AliasedGroup
 from .odoo_config import MANIFEST, current_version, customs_dir
 from .tools import _execute_sql
+from .tools import _get_setting
 
 
 @cli.group(cls=AliasedGroup)
@@ -18,7 +21,7 @@ def talk(config):
 
 
 @talk.command()
-@click.argument("name", required=False)
+@click.argument("name", required=False, nargs=-1)
 @click.option("-M", "--module")
 @click.option("-m", "--model")
 @pass_config
@@ -29,11 +32,11 @@ def xmlids(config, name, module, model):
         where += f" AND model = '{model}'"
     if module:
         where += f" AND module = '{model}'"
-    if name:
+    for name in name:
         where += (
-            f" and (model ilike '%{name}%' or "
+            f" and ( (model ilike '%{name}%' or "
             f"name ilike '%{name}%' or "
-            f"module ilike '%{name}%')"
+            f"module ilike '%{name}%'))"
         )
     rows = _execute_sql(
         conn,
@@ -66,6 +69,15 @@ def deactivate_field_in_views(config, field):
         )
 
 
+@talk.command()
+@click.argument("name", required=True)
+@pass_config
+@click.pass_context
+def get_config_parameter(ctx, config, name):
+    conn = config.get_odoo_conn()
+    click.secho(_get_setting(conn, name))
+
+
 @talk.command()
 @click.argument("name", required=True)
 @click.option("-Q", "--quick", is_flag=True)
@@ -319,7 +331,7 @@ def users(config, login):
         return_columns=False,
     )
 
-    cols = ["login", "name"]
+    cols = ["login", "name", "user_id"]
     click.secho(
         tabulate(rows, cols, tablefmt="fancy_grid"),
         fg="yellow",
@@ -358,4 +370,118 @@ def fields(config, model, field):
     )
 
 
+@talk.command()
+@click.option("-i", "--interval", default=5, type=int)
+@pass_config
+def queuejobs(config, interval):
+    conn = config.get_odoo_conn()
+    last_data, last_time = None, None
+    averages = {}
+    while True:
+        rows = _execute_sql(
+            conn,
+            sql=(
+                "SELECT count(*) as count, state "
+                "FROM queue_job "
+                "GROUP BY state "
+                "UNION "
+                "SELECT count(*), 'total' FROM queue_job"
+            ),
+            fetchall=True,
+            return_columns=True,
+        )
+        rows = list(rows)
+        rows[1] = sorted(rows[1], key=lambda x: x[1] == "total" and "zzzzzzzz" or x[1])
+        data = {x[1]: x[0] for x in rows[1]}
+
+        click.secho(tabulate(rows[1], rows[0], tablefmt="fancy_grid"), fg="yellow")
+        if last_data:
+            click.secho("Changes: ")
+            now = arrow.get()
+            avg_rows = []
+            for state, v in data.items():
+                diff = data.get(state, 0) - last_data.get(state, 0)
+                seconds = round((now - last_time).total_seconds())
+                if seconds:
+                    diff_per_second = round(abs(diff / seconds), 1)
+                else:
+                    diff_per_second = 0
+                averages.setdefault(state, [])
+                averages[state].append(diff_per_second)
+                avg_diff_per_second = round(
+                    sum(averages[state]) / len(averages[state]), 1
+                )
+            click.secho(
+                tabulate(
+                    avg_rows,
+                    ["state", "items", "items per second"],
+                    tablefmt="fancy_grid",
+                ),
+                fg="blue",
+            )
+            # click.secho(f"{state}: {diff} with {avg_diff_per_second}/s")
+
+        time.sleep(interval)
+        last_data = data
+        last_time = arrow.get()
+
+
+@talk.command()
+@click.option("-M", "--module")
+@click.argument("model", required=True)
+@click.argument("name", required=False)
+@pass_config
+def views(config, name, module, model):
+    conn = config.get_odoo_conn()
+
+    def _get_xmlid(id):
+        where = f"model = 'ir.ui.view' and res_id={id}"
+        sql = (
+            "SELECT module||'.'|| name as xmlid, model, res_id from ir_model_data "
+            f"where {where} "
+        )
+        rows = _execute_sql(
+            conn,
+            sql=sql,
+            fetchall=True,
+            return_columns=False,
+        )
+        if rows:
+            return rows[0][0]
+        return None
+
+    where = f"model = '{model}'"
+    where += "AND inherit_id is null"
+    rows = _execute_sql(
+        conn,
+        sql=f"select id, name from ir_ui_view where {where}",
+        fetchall=True,
+        return_columns=True,
+    )
+
+    rows = list(rows)
+    rows2 = []
+    for row in rows[1]:
+        row = list(row)
+        row.append(_get_xmlid(row[0]))
+        rows2.append(row)
+    rows[0] = list(rows[0])
+    rows[0].append("xmlid")
+    click.secho(tabulate(rows2, rows[0], tablefmt="fancy_grid"), fg="yellow")
+
+
+@talk.command()
+@pass_config
+@click.pass_context
+def set_remote_keys(ctx, config):
+    Commands.invoke(
+        ctx,
+        "odoo-shell",
+        command=[
+            'env["res.users"].set_remote_keys();',
+            "env.cr.commit()",
+        ],
+    )
+
+
 Commands.register(progress)
