@@ -1,8 +1,14 @@
+from packaging.specifiers import SpecifierSet
+from packaging.version import Version
+from packaging.requirements import Requirement
+from packaging import markers
+
+
 import arrow
 import pprint
 import json
 import click
-import iscompatible
+from . import iscompatible
 from pathlib import Path
 from copy import deepcopy
 import pickle
@@ -46,19 +52,11 @@ import pprint
 from lxml import etree
 import subprocess
 
-try:
-    import xmlrpclib
-except Exception:
-    import xmlrpc
-    from xmlrpc import client as xmlrpclib
 import inspect
 import sys
 
 LANG = os.getenv("ODOO_LANG", "de_DE")  # todo from environment
-host = "http://localhost:8069"
 
-username = "admin"
-pwd = "1"
 
 name_cache = {}
 remark_about_missing_module_info = set()
@@ -77,16 +75,7 @@ class NotInAddonsPath(Exception):
     pass
 
 
-def exe(*params):
-    config = get_settings()
-
-    def login(username, password):
-        socket_obj = xmlrpclib.ServerProxy("%s/xmlrpc/common" % (host))
-        return socket_obj.login(config["DBNAME"], username, password)
-
-    uid = login(username, pwd)
-    socket_obj = xmlrpclib.ServerProxy("%s/xmlrpc/object" % (host))
-    return socket_obj.execute(config["DBNAME"], uid, pwd, *params)
+from .tools import exe
 
 
 def delete_qweb(config, modules):
@@ -174,10 +163,16 @@ class DBModules(object):
         pass
 
     @classmethod
-    def check_if_all_modules_from_install_are_installed(clazz):
+    def check_if_all_modules_from_install_are_installed(clazz, check_func):
+        res = set()
         for module in get_modules_from_install_file():
             if not clazz.is_module_installed(module):
-                yield module
+                res.add(module)
+
+        if check_func:
+            check_func(res)
+
+        return list(res)
 
     @classmethod
     def abort_upgrade(clazz):
@@ -186,7 +181,7 @@ class DBModules(object):
             UPDATE ir_module_module SET state = 'uninstalled' WHERE state = 'to install';
         """
         with get_conn_autoclose() as cr:
-            if table_exists(cr, 'ir_module_module'):
+            if table_exists(cr, "ir_module_module"):
                 _execute_sql(cr, SQL)
 
     @classmethod
@@ -353,7 +348,7 @@ def make_customs(config, ctx, path, version, odoosh):
     if not path.exists():
         abort("Path does not exist: {}".format(path))
     elif list(path.glob("*")):
-        files = [x for x in path.glob("*") if x.name != '.git']
+        files = [x for x in path.glob("*") if x.name != ".git"]
         if files:
             if not config.force:
                 abort("Path is not empty: {}".format(path))
@@ -898,7 +893,7 @@ class Modules(object):
 
         return list(result)
 
-    def get_all_external_dependencies(self, modules):
+    def get_all_external_dependencies(self, modules, python_version):
         global_data = {"pip": []}
         for module_name in modules:
             module = Module.get_by_name(module_name)
@@ -923,38 +918,67 @@ class Modules(object):
             else:
                 extract_deps(module.manifest_dict.get("external_dependencies", {}))
 
-        global_data["pip"] = self.resolve_pydeps(set(global_data["pip"]))
+        global_data["pip"] = self.resolve_pydeps(
+            set(global_data["pip"]), python_version
+        )
         return global_data
 
-    def resolve_pydeps(self, pydeps):
-        pydeps = list(set(pydeps))
-        libnames = [_extract_python_libname(x) for x in pydeps]
-        result = set()
+    def _filter_requirements(self, pydeps, python_version):
+        assert isinstance(python_version, tuple)
+        str_python_version = '.'.join(map(str, python_version[:2]))
+        environment = {
+            "python_version": str_python_version,
+            "sys_platform": sys.platform
+        }
 
         # keep highest version and or leaveout loosers
-        def _map(x):
-            if x:
-                arr = iscompatible.parse_requirements(x)
-            else:
-                arr = []
-            for arr in arr:
-                if arr:
-                    arr = list(arr)
-                    arr[1] = iscompatible.string_to_tuple(arr[1])
-                yield tuple(arr)
-
-        parsed_requirements = []
-        for inst in pydeps:
-            libname = _extract_python_libname(inst)
-            if "@" in inst:
+        def _filter(x):
+            if not x:
+                return
+            if "@" in x:
                 # concrete url like "pymssql@git+https://github.com/marcwimmer/pymssql"
-                parsed_requirements.append(inst)
-            else:
-                for parsed in [
-                    _map(x) for x in pydeps if _extract_python_libname(x) == libname
-                ]:
-                    for x in parsed:
-                        parsed_requirements.append((libname, x))
+                return x
+            x = x.split("#")[0].strip()
+            if not x:
+                return
+            for extra in x.split(";")[1:]:
+                expr = markers.Marker(extra)
+                res = expr.evaluate(environment)
+                if not res:
+                    return
+            return STRIP(x)
+
+        def STRIP(dep):
+            res = dep.split("#")[0].split(";")[0].strip()
+            if res.endswith(".*"):
+                # xlwt==1.3.*
+                res = res[:-2]
+            return res
+
+        def _make_tuples(dep):
+            libname = _extract_python_libname(dep)
+            if not libname:
+                return []
+            try:
+                reqs = iscompatible.parse_requirements(dep)
+            except Exception as ex:
+                raise
+            reqs = list(reqs)
+            if reqs:
+                for i in range(len(reqs)):
+                    try:
+                        reqs[i] = list(reqs[i])
+                        reqs[i][1] = iscompatible.string_to_tuple(reqs[i][1])
+                    except:
+                        raise
+            return libname, tuple(reqs)
+
+        reqs = {}
+        for dep in list(filter(_filter, pydeps)):
+            dep = STRIP(dep)
+            print(dep)
+            libname, version = _make_tuples(dep)
+            reqs.setdefault(libname, []).append(version)
         """
         parsed_requirements ilike
         [
@@ -962,19 +986,28 @@ class Modules(object):
         ('>=', '1.2.0')
         ]
         """
+        return reqs
 
+    def _pydeps_filter_best_fit(self, pydeps):
+        result = set()
         allowed = ["==", ">="]
         unallowed = [
-            x
-            for x in parsed_requirements
-            if not isinstance(x, str) and x[1][0] not in allowed
+            x for x in pydeps if not isinstance(x, str) and x[1][0] not in allowed
         ]
         if unallowed:
             raise Exception(f"Unhandled: {unallowed} - only {allowed} allowed")
 
-        for libname in libnames:
+        for libname in pydeps.keys():
             # mixed == and >=
-            reqs = list(set([x[1] for x in parsed_requirements if x[0] == libname]))
+            reqs = pydeps.get(libname, [])
+
+            # handle case: reqs = [()]
+            unspecific_ones = [x for x in reqs if not x]
+            if len(unspecific_ones) == len(reqs):
+                result.add(libname)
+                continue
+            reqs = [x for x in reqs if x]
+
             ge = sorted([x for x in reqs if x[0] == ">="], key=lambda x: x[1])
             gt = sorted(
                 [x for x in reqs if x[0] == ">"], key=lambda x: x[1]
@@ -1006,7 +1039,20 @@ class Modules(object):
             elif ge:
                 result.add(f"{libname}{ge[-1][0]}{'.'.join(map(str, ge[-1][1]))}")
             else:
-                result.add(libname)
+                if len(reqs) == 1:
+                    result.add(
+                        f"{libname}{reqs[0][-1][0]}{'.'.join(map(str, reqs[0][-1][1]))}"
+                    )
+                else:
+                    result.add(libname)
+        return list(sorted(result))
+
+    def resolve_pydeps(self, pydeps, python_version):
+        pydeps = list(set(pydeps))
+
+        parsed_requirements = self._filter_requirements(pydeps, python_version)
+        result = self._pydeps_filter_best_fit(parsed_requirements)
+
         return list(result)
 
 
@@ -1072,6 +1118,23 @@ class Module(object):
         else:
             self.name = self._manifest_path.parent.name
 
+    @property
+    def descendants(self):
+        res = []
+        mods = Modules()
+        modules = mods.get_all_modules_installed_by_manifest()
+        all_modules = [self.get_by_name(x) for x in modules]
+        for check in all_modules:
+            if self.name in [
+                x.name for x in mods.get_module_flat_dependency_tree(check)
+            ]:
+                res.append(check)
+        return res
+
+    @property
+    def is_customs(self):
+        return self.path.parts[0] not in ["odoo", "enterprise", "themes"]
+
     @property
     def exists(self):
         return bool(self.path)
@@ -1385,6 +1448,8 @@ class Module(object):
                 continue
             if ".git" in file.parts:
                 continue
+            if file.parts[0].startswith("."):
+                continue
             # relative to module path
             yield file
 
@@ -1487,11 +1552,13 @@ class Module(object):
         if current_version() > 14.0:
             if "qweb" in mod:
                 mod.setdefault("assets", {})
-                mod["assets"].setdefault("web.assets_qweb", [])
-                mod["assets"]["web.assets_qweb"] += mod.pop("qweb")
-                mod["assets"]["web.assets_qweb"] = list(
-                    sorted(set(mod["assets"]["web.assets_qweb"]))
-                )
+                if current_version() == 15.0:
+                    key = "web.assets_qweb"
+                else:
+                    key = "web.assets_backend"
+                mod["assets"].setdefault(key, [])
+                mod["assets"][key] += mod.pop("qweb")
+                mod["assets"][key] = list(sorted(set(mod["assets"][key])))
 
         # now sort again by inspecting file content - if __openerp__.sequence NUMBER is found, then
         # set this index; reason: some times there are wizards that reference forms and vice versa
