@@ -1,3 +1,5 @@
+import platform
+import inspect
 import socket
 import uuid
 import time
@@ -44,6 +46,59 @@ import inspect
 from copy import deepcopy
 from passlib.context import CryptContext
 
+try:
+    import xmlrpclib
+except Exception:
+    import xmlrpc
+    from xmlrpc import client as xmlrpclib
+
+
+def get_calling_function_variables():
+    # Get the current stack frame
+    current_frame = inspect.currentframe()
+    # Get the frame of the caller (one level up in the stack)
+    caller_frame = current_frame.f_back.f_back
+    # Get the local variables of the caller
+    caller_locals = caller_frame.f_locals
+    return caller_locals
+
+
+def odoorpc(config):
+    PROXY_PORT = config.PROXY_PORT
+    import odoorpc
+
+    if "ODOO_USER" not in os.environ:
+        click.secho(
+            "Tip: set ODOO_USER and ODOO_PASSWORD in your environment", fg="yellow"
+        )
+
+    odoo = odoorpc.ODOO("localhost", port=PROXY_PORT)
+    username = os.getenv("ODOO_USER", "admin")
+    pwd = os.getenv("ODOO_PASSWORD", config.DEFAULT_DEV_PASSWORD or "admin")
+    odoo.login(config.DBNAME, username, pwd)
+    odoo.config["auto_context"] = False  # often context probided by self
+    return odoo
+
+
+def exe(*params, **kwparams):
+    vars = get_calling_function_variables()
+    config = vars["config"]
+    PROXY_PORT = config.PROXY_PORT or 8069
+    host = f"http://localhost:{PROXY_PORT}"
+    username = "admin"
+    pwd = config.DEFAULT_DEV_PASSWORD or "admin"
+    from .odoo_config import get_settings
+
+    config = get_settings()
+
+    def login(username, password):
+        socket_obj = xmlrpclib.ServerProxy("%s/xmlrpc/common" % (host))
+        return socket_obj.login(config["DBNAME"], username, password)
+
+    uid = login(username, pwd)
+    socket_obj = xmlrpclib.ServerProxy("%s/xmlrpc/object" % (host))
+    return socket_obj.execute(config["DBNAME"], uid, pwd, *params, **kwparams)
+
 
 class DBConnection(object):
     def __init__(self, dbname, host, port, user, pwd):
@@ -452,16 +507,16 @@ def _set_default_envs(env):
     return env
 
 
-def __dc(config, cmd, env={}):
+def __dc(config, cmd, env={}, profile="auto"):
     ensure_project_name(config)
-    c = __get_cmd(config) + cmd
+    c = __get_cmd(config, profile=profile) + cmd
     env = _set_default_envs(env)
     return subprocess.check_call(c, env=_merge_env_dict(env))
 
 
-def __dc_out(config, cmd, env={}):
+def __dc_out(config, cmd, env={}, profile="auto"):
     ensure_project_name(config)
-    c = __get_cmd(config) + cmd
+    c = __get_cmd(config, profile=profile) + cmd
     env = _set_default_envs(env)
     return subprocess.check_output(c, env=_merge_env_dict(env))
 
@@ -491,6 +546,8 @@ def __dcrun(
     returncode=False,
     pass_stdin=None,
     returnproc=False,
+    detached=False,
+    name=None,
 ):
     ensure_project_name(config)
     env = _set_default_envs(env)
@@ -498,6 +555,10 @@ def __dcrun(
     cmd = ["run"]
     if not interactive:
         cmd += ["-T"]
+    if detached:
+        cmd += ["-d"]
+    if name:
+        cmd += ["--name", name]
     cmd += ["--rm"]
     for k, v in env.items():
         cmd += ["-e{}={}".format(k, v)]
@@ -509,6 +570,7 @@ def __dcrun(
         if pass_stdin:
             optional_params["input"] = pass_stdin
             optional_params["universal_newlines"] = True
+            cmd.insert(len(cmd) - 2, "-T")
         else:
             optional_params["stdin"] = sys.stdin
         return subprocess.run(cmd, check=True, **optional_params)
@@ -595,8 +657,23 @@ def __safeget(array, index, exception_on_missing, file_options=None):
     return array[index]
 
 
-def __get_cmd(config):
-    cmd = config.commands["dc"]
+def get_docker_version():
+    docker = search_env_path("docker")
+    version = subprocess.check_output([docker, "--version"], encoding="utf8").strip()
+    # support for stripping debian specific version suffixes
+    version = re.sub("\+dfsg[0-9]*,", "", version)
+    # continue with normal version parsing
+    version = list(map(int, version.split(" ")[2].replace(",", "").split(".")))
+    return version
+
+
+def __get_cmd(config, profile="auto"):
+    if get_docker_version()[0] >= 26:
+        cmd = config.commands["dc2"]
+    else:
+        cmd = config.commands["dc"]
+    if profile:
+        cmd += ["--profile", profile]
     cmd = [os.path.expandvars(x) for x in cmd]
     return cmd
 
@@ -719,6 +796,11 @@ def _get_user_primary_group(UID):
     return subprocess.check_output([id, "-gn", str(UID)], encoding="utf8").strip()
 
 
+def verbose(txt):
+    if os.getenv("WODOO_VERBOSE") == "1":
+        click.secho(txt, fg="gray")
+
+
 def __try_to_set_owner(UID, path, abort_if_failed=True, verbose=False):
     primary_group = _get_user_primary_group(UID)
     find_command = f"find '{path}' -not -type l -not -user {UID}"
@@ -874,13 +956,17 @@ def remove_webassets(conn):
         f"delete from ir_attachment where name ilike '%.assets_%.js' and res_model = 'ir.ui.view'",
     ]
     try:
-        for query in queries:
-            try:
-                click.secho(query, fg="grey")
-                cr.execute(query)
-                conn.commit()
-            except:
-                continue
+        with click.progressbar(
+            queries, label="Executing queries", length=len(queries)
+        ) as bar:
+            for query in queries:
+                try:
+                    bar.update(1, query)
+                    cr.execute(query)
+                    conn.commit()
+                except Exception as ex:
+                    click.secho(str(ex), fg="red")
+                    continue
     finally:
         cr.close()
         conn.close()
@@ -1095,9 +1181,12 @@ def split_hub_url(config):
             fg="yellow",
         )
         return None
-    username, password = url.split(":", 1)
-    password = password.split("@")[0]
-    url = url.split("@")[1]
+    if "@" in url:
+        username, password = url.split(":", 1)
+        password = password.split("@")[0]
+        url = url.split("@")[1]
+    else:
+        username, password = "", ""
     url, prefix = url.split("/", 1)
     return {
         "url": url,
@@ -1177,17 +1266,26 @@ def get_directory_hash(path):
     click.secho(f"Calculating hash for {path}", fg="yellow")
     # "-N required because absolute path is used"
     remove_pycs(path)
-    hex = (
-        subprocess.check_output(
-            ["dtreetrawl", "-N", "--hash", "-R", path], encoding="utf8"
+    try:
+        hex = (
+            subprocess.check_output(
+                ["dtreetrawl", "-N", "--hash", "-R", path], encoding="utf8"
+            )
+            .strip()
+            .split(" ")[0]
+            .strip()
         )
-        .strip()
-        .split(" ")[0]
-        .strip()
-    )
+    except Exception as ex:
+        if onM1():
+            return ""
+        raise
     return hex
 
 
+def onM1():
+    return platform.machine() == "aarch64"
+
+
 def git_diff_files(path, commit1, commit2):
     params = [
         "git",
@@ -1376,7 +1474,7 @@ def _get_default_project_name(restrict):
     if customs_root:
         root = Path(customs_root)
         if (root / "MANIFEST").exists():
-            return root.name
+            return root.name.replace(".", "_")
     raise NoProjectNameException("No default project name could be determined.")
 
 
@@ -1442,7 +1540,7 @@ def _write_file(file, content):
     return False
 
 
-def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
+def _make_sure_module_is_installed(ctx, config, modulename, repo_url, update=False):
     from .module_tools import DBModules
     from .module_tools import Modules, Module
     from .odoo_config import MANIFEST
@@ -1452,18 +1550,13 @@ def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
     from gimera.gimera import add as gimera_add
     from gimera.gimera import apply as gimera_apply
 
-    state = DBModules.get_meta_data(modulename)
-    if state and state["state"] == "installed":
-        return
-    import pudb
-
-    pudb.set_trace()
+    dest_path = Path("addons_wodoo") / modulename
 
     manifest = MANIFEST()
     try:
         Module.get_by_name(modulename)
     except (KeyError, NotInAddonsPath):
-        path = Path("addons_wodoo") / modulename
+        path = dest_path
         if not path.exists():
             ctx.invoke(
                 gimera_add,
@@ -1472,7 +1565,7 @@ def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
                 branch=str(current_version()),
                 type="integrated",
             )
-            ctx.invoke(gimera_apply, repos=[str(path)])
+            ctx.invoke(gimera_apply, repos=[str(path)], no_auto_commit=True)
 
             # if not yet there, then pack into "addons_framework"
             addons_paths = manifest.get("addons_paths", [])
@@ -1480,13 +1573,25 @@ def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
             if str(path) not in addons_paths:
                 addons_paths += [str(path)]
                 manifest["addons_paths"] = addons_paths
-
+    if update:
+        ctx.invoke(
+            gimera_apply, repos=[str(dest_path)], update=True, no_auto_commit=True
+        )
     install = manifest.get("install", [])
     if modulename not in install:
         install += [modulename]
     manifest["install"] = install
 
+    addons_path = str(dest_path)
+    addons_paths = manifest["addons_paths"]
+    if addons_path not in addons_paths:
+        addons_paths.append(addons_path)
+    manifest["addons_paths"] = addons_paths
+
     manifest.rewrite()
+    state = DBModules.get_meta_data(modulename)
+    if state and state["state"] == "installed":
+        return
 
     Commands.invoke(
         ctx,
@@ -1630,3 +1735,37 @@ def force_input_hostname():
     value = click.prompt(f"Please type the hostname of the machine again [{hostname}]")
     if value != hostname:
         abort(f"You typed {value} but {hostname} was expected.")
+
+
+def start_postgres_if_local(ctx, config):
+    from .cli import cli, pass_config, Commands
+
+    if config.run_postgres:
+        Commands.invoke(ctx, "up", machines=["postgres"], daemon=True)
+        Commands.invoke(ctx, "wait_for_container_postgres", missing_ok=True)
+
+
+def update_setting(config, name, value):
+    from .myconfigparser import MyConfigParser
+
+    configparser = MyConfigParser(config.files["project_settings"])
+    configparser[name] = value
+    configparser.write()
+
+
+def update_docker_service(config, service_name, service_content):
+    import yaml
+
+    filepath = config.files["project_docker_compose.home.project"]
+    click.secho(f"Writing to {filepath}", fg="yellow")
+    content = __read_file(filepath)
+    compose = yaml.safe_load(content)
+    compose["services"][service_name] = service_content
+    content = yaml.dump(compose, default_flow_style=False)
+    __write_file(filepath, content)
+
+
+def _parse_yaml(content):
+    import yaml
+
+    return yaml.safe_load(content)
